const functions = require('firebase-functions');
const admin = require('firebase-admin');
const iot = require('@google-cloud/iot');

admin.initializeApp(functions.config().firebase);

const db = admin.database();

//global.resources = "Test";
var client = new iot.v1.DeviceManagerClient({});

var project = '<PROJECT-NAME>';
var registry = '<REGISTRY-NAME>';
var topic = '<TOPCIC-NAME>';
var region = 'us-central1';

/** Cloud Functions */

exports.receiveGPS = functions.pubsub
  .topic(topic)
  .onPublish((message, context) => {
    const attributes = message.attributes;
    const payload = message.json;
    const deviceId = attributes['deviceId'];
    var path;
    let data = {
      deviceId: deviceId,
      timestamp: context.timestamp
    };
    if ('latitude' in payload.data) {
      data.latitude = payload.data.latitude;
      data.longitude = payload.data.longitude;
      data.altitude = payload.data.altitude;
      data.speed = payload.data.speed;
      data.heading = payload.data.heading;
      data.datetime_year = payload.data.datetime_year;
      data.datetime_month = payload.data.datetime_month;
      data.datetime_day = payload.data.datetime_day;
      data.datetime_hour = payload.data.datetime_hour;
      data.datetime_minute = payload.data.datetime_minute;
      data.datetime_seconds = payload.data.datetime_seconds;
      data.path_id = payload.data.path_id;


      db.ref(`/clicks/${deviceId}`).once("value", function (snapshot) {
        if (snapshot.val().pathID !== data.path_id) {
          path = snapshot.toJSON();
          for (k in path) {
            delete path[k].timestamp;
          }
          handlePOST(path, deviceId);
        }
      });

      return Promise.all([
        storeDataInFirebase(data)
      ]);
    }
    else if ('uniqueMarkerID' in payload.data) {
      data.uniqueMarkerID = payload.data.uniqueMarkerID;

      return Promise.all([
        removeFinishedMarkerFirebase(data)
      ]);
    }
    else {
      console.log("Data is corrupted: ", data);
      return;
    }
  });


/** Helper functions */

function handlePOST(data, device) {
  let binaryData = Buffer.from(JSON.stringify(data));
  formattedName = client.devicePath(project,
    region, registry, device);
  request = {
    name: formattedName,
    binaryData: binaryData,
  }
  client.modifyCloudToDeviceConfig(request)
    .catch(err => {
      console.error(err);
    });
}

function storeDataInFirebase(data) {
  let dbRef = db.ref(`/gps91/${data.deviceId}`);
  return dbRef.set({
    path_id: data.path_id,
    lastTimestamp: data.timestamp,
    latitude: data.latitude,
    longitude: data.longitude,
    altitude: data.altitude,
    speed: data.speed,
    heading: data.heading,
    year: data.datetime_year,
    month: data.datetime_month,
    day: data.datetime_day,
    hour: data.datetime_hour,
    minute: data.datetime_minute,
    seconds: data.datetime_seconds
  });
}

function removeFinishedMarkerFirebase(data) {
  let markerToRemove = data.uniqueMarkerID;
  let dbRef = db.ref(`/clicks/${data.deviceId}/${markerToRemove}`);
  if (dbRef == db.ref('clicks')) {
    console.log("Tried to remove whole database. Cancelling command.");
    return;
  } else {
    dbRef.remove();
    let ref = db.ref(`clicks/${data.deviceId}/`);
    ref.once("value", function (snapshot) {
      uniquePathID = generateRandomID();
      uniquePathID =
        FormatNumberLength(Object.values(snapshot.val()).length-1, 2)
          .concat(uniquePathID);
      ref.child('pathID').set(uniquePathID);
      path = snapshot.toJSON();
      for (k in path) {
        delete path[k].timestamp;
      }
      delete path.markerToRemove;
      handlePOST(path, data.deviceId);
    }, function (errorObject) {
      console.log("The read failed: " + errorObject.code);
    });
  }
}

var generateRandomID = function () {
  return '_' + Math.random().toString(36).substr(2, 7);
};

function FormatNumberLength(num, length) {
  var r = "" + num;
  while (r.length < length) {
    r = "0" + r;
  }
  return r;
}